#!/usr/bin/env python

from __future__ import division

import pygtk
pygtk.require('2.0')
import gtk

import numpy as np
import matplotlib.cm as cm
import matplotlib.mlab as mlab 
from matplotlib.figure import Figure
from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg

import Magnet

class FunctionEntry(gtk.Entry):
    def __init__(self, default="0.0"):
        gtk.Entry.__init__(self)
        self.set_text(default)

class NumberEntry(gtk.Entry):
    def __init__(self, default="0.0"):
        gtk.Entry.__init__(self)
        self.old_text = default
        self.set_text(self.old_text)
        self.connect('changed', self.on_changed)

    def on_changed(self, *args):
        text = self.get_text()
        try:
            float(text)
            self.old_text = text
        except ValueError:
            if text == '' or text == '-':
                self.old_text = text
            else:
                text = self.old_text
        self.set_text(text)

    def get_value(self):
        return float(self.get_text())

class MagnetInput(gtk.HBox):

    def __init__(self):
        gtk.HBox.__init__(self, False, 0)

        def addEntry(text, value, width=3, EntryClass=NumberEntry):
            box = gtk.VBox(False, 0)
            label = gtk.Label(text)
            label.set_justify(gtk.JUSTIFY_FILL)
            box.pack_start(label, True, True, 0)
            label.show()
            entry = EntryClass(str(value))
            entry.set_width_chars(width)
            box.pack_start(entry, True, True, 0)
            entry.show()
            self.pack_start(box)
            box.show()
            return (box, entry)

        dummy, self.MEntry = addEntry("M_z=", "r", 10, FunctionEntry)
        dummy, self.radiusEntry = addEntry("radius", 0.05)
        dummy, self.heightEntry = addEntry("height:", 0.01)
        dummy, self.z_0Entry = addEntry("Orig. disp.:", 0)
        dummy, self.granEntry = addEntry("Cyl. gran.:", "10")
        dummy, self.cylnoEntry = addEntry("No. of cyls:", "10")
        

        self.remove_button = gtk.Button("-")
        self.remove_button.connect("clicked", self._remove_button_callback)
        self.pack_start(self.remove_button, True, False, 0)
        self.remove_button.show()
        self._callbacks = []

    def _remove_button_callback(self, button, func_data = None):
        for c in self._callbacks:
            c(self)

    def remove_button_connect(self, callback, func_data = None):
        self._callbacks.append(callback)

    @property
    def M(self):
        return lambda r: eval(self.MEntry.get_text())

    @property
    def radius(self):
        return self.radiusEntry.get_value()

    @property
    def height(self):
        return self.heightEntry.get_value()

    @property
    def z_0(self):
        return self.z_0Entry.get_value()

    @property
    def gran(self):
        return int(self.granEntry.get_value())

    @property
    def cylno(self):
        return int(self.cylnoEntry.get_value())

class MagnetList(gtk.VBox):

    def __init__(self):
        gtk.VBox.__init__(self, False, 0)
        self._list = []

    def addMagnet(self):
        m = MagnetInput()
        self._list.append(m)
        self.pack_start(m, True, True, 0)
        m.remove_button_connect(self.removeMagnet)
        m.show()
        return m

    def removeMagnet(self, m, data=None):
        self.remove(m)
        self._list.remove(m)

    class MIter(object):
        def __init__(self, parent):
            self.parent = parent
            self.index = 0

        def next(self):
            t = self.index
            self.index += 1
            if t >= len(self.parent._list):
                raise StopIteration
            return self.parent._list[t]

    def __iter__(self):
        return self.MIter(self)


class Cylinder(object):

    def __init__(self, height, radius, sign, z_0, gran):
        self.height = height
        self.radius = radius
        self.sign = sign
        self.z_0 = z_0
        self.gran = gran

        
class MagnetFun(object):

    def draw_r0(self, r0, max_mag):
        ax = self.constraxes
        ax.cla()
        plotdata = self.f_mag[self.r==np.nanmin(self.r[self.r>=r0])]
        plotdata = plotdata.copy()
        plotdata[plotdata>max_mag] = float("nan")
        ax.plot(self.Z, plotdata)
        ax.set_title('Force magnitude at r==%f' % r0)
        self.constrcanvas.draw()

    def draw_z0(self, z0, max_mag):
        ax = self.constzaxes
        ax.cla()
        plotdata = self.f_mag[self.z==np.nanmin(self.z[self.z>=z0])]
        plotdata = plotdata.copy()
        plotdata[plotdata>max_mag] = float("nan")
        ax.plot(self.R, plotdata)
        ax.set_title('Force magnitude at z==%f' % z0)
        self.constzcanvas.draw()

    def onclick(self, event):
        self.draw_r0(event.xdata, 2**self.max_adj.get_value())
        self.draw_z0(event.ydata, 2**self.max_adj.get_value())

    def draw(self, *arg):
        max_mag = self.max_adj.get_value()
#        f_mag = self.f_mag.copy()
#        f_mag = np.ma.array(f_mag)
        self.f_mag.mask = False
        self.f_mag.mask[self.f_mag.data>2**max_mag] = True
        ax = self.mainaxes
        ax.cla()
        print 2**max_mag
        tmp = np.linspace(0,1,20)**7
        lvls = tmp*2**max_mag*1000
        print lvls
        self.CS = ax.contour(self.r,self.z,self.f_mag*1000, lvls)
#        self.CS2 = ax.contourf(self.r,self.z,self.f_mag*1000,20,cmap=cm.bone)
        ax.clabel(self.CS, fontsize=10)
        ax.set_title('Kraft i mN')
        ax.set_ylabel('Vertikal avstand (m)')
        ax.set_xlabel('Radiell avstand (m)')
        self.maincanvas.draw()

    def update_graph(self, widget, data=None):
        new_mlist= []
        for magnet in self.mlist:
            for n in range(magnet.cylno):
                r_n = (n+1)*magnet.radius/magnet.cylno
                M_r_n = magnet.M(r_n)
                if n == magnet.cylno - 1:
                    M_r_npdn = 0
                else:
                    M_r_npdn = magnet.M((n+2)*magnet.radius/magnet.cylno)
                M_frac = M_r_n-M_r_npdn
                print magnet.height, r_n, M_frac, magnet.z_0, magnet.gran
                new_mlist.append(Cylinder(height=magnet.height,
                                          radius=r_n,
                                          sign=M_frac,
                                          z_0=magnet.z_0,
                                          gran=magnet.gran))
        self.f_mag = np.ma.array(Magnet.BuildForceField(new_mlist, self.r, self.z))
        mn = max(np.log2(np.nanmin(self.f_mag.data)), np.log2(1e-6))
        mx = np.log2(np.nanmax(self.f_mag.data))
        self.max_adj.set_all((mn+mx)/2, mn, mx)
        self.draw()

    def add_magnet(self, widget, data=None):
        self.mlist.addMagnet()

    def save_plot(self, widget, data=None):
        self.maincanvas.print_png("ut.png")

    def delete_event(self, widget, event, data=None):
        gtk.main_quit()
        return False

    def __init__(self):
        self.R = np.linspace(-5e-2,5e-2,401)
        self.Z = np.linspace(-2e-2,5e-2,401)
        self.r, self.z = np.meshgrid(self.R, self.Z)

        self.fig_z0 = None
        self.fig_r0 = None

        self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
        self.window.set_title("Magnets!")
        self.window.set_icon_from_file("Magnet.jpg")
        self.window.connect("delete_event", self.delete_event)
        self.window.set_border_width(10)

        self.mainpane = gtk.HPaned()
        self.window.add(self.mainpane)
        self.mainpane.show()

        self.mainfigpane = gtk.VPaned()
        self.mainpane.pack1(self.mainfigpane)
        self.mainfigpane.show()

        self.mainfig = Figure(figsize=(4,3), dpi=100)
        self.mainaxes = self.mainfig.add_subplot(111)
        self.maincanvas = FigureCanvasGTKAgg(self.mainfig)
        self.maincanvas.mpl_connect('button_press_event', self.onclick)
        self.maincanvas.size(400,400)
        self.mainfigpane.add1(self.maincanvas)
        self.maincanvas.show()


        self.box1 = gtk.VBox(False,0)
        self.mainfigpane.pack2(self.box1, False, False)

        self.mlist = MagnetList()
        self.box1.pack_start(self.mlist, False, False, 0)
        self.mlist.show()

        self.max_adj = gtk.Adjustment(0.0, -101.0, 101.0, 1.0)
        self.max_scale = gtk.HScale(self.max_adj)
        self.max_scale.set_digits(4)
        self.max_scale.set_update_policy(gtk.UPDATE_DISCONTINUOUS)
        self.max_scale.connect("value_changed", self.draw)
        self.box1.pack_start(self.max_scale)
        self.max_scale.show()

        self.box2 = gtk.HBox(False,0)
        self.draw_button = gtk.Button("Draw")
        self.draw_button.connect("clicked", self.update_graph)
        self.box2.pack_start(self.draw_button, False, False, 10)
        self.draw_button.show()
        self.add_button = gtk.Button("Add")
        self.add_button.connect("clicked", self.add_magnet)
        self.box2.pack_start(self.add_button, False, False, 10)
        self.add_button.show()
        self.save_button = gtk.Button("Save")
        self.save_button.connect("clicked", self.save_plot)
        self.box2.pack_start(self.save_button, False, False, 10)
        self.save_button.show()
        self.box1.pack_start(self.box2, False, False, 0)
        self.box2.show()

        self.box1.show()

        self.subfig = gtk.VBox(True,0)
        self.mainpane.pack2(self.subfig)
        self.subfig.show()

        self.constzfig = Figure(figsize=(4,3), dpi=100)
        self.constzaxes = self.constzfig.add_subplot(111)
        self.constzcanvas = FigureCanvasGTKAgg(self.constzfig)
        self.constzcanvas.size(100,100)
        self.subfig.pack_start(self.constzcanvas, True, True, 0)
        self.constzcanvas.show()

        self.constrfig = Figure(figsize=(4,3), dpi=100)
        self.constraxes = self.constrfig.add_subplot(111)
        self.constrcanvas = FigureCanvasGTKAgg(self.constrfig)
        self.constrcanvas.size(100,100)
        self.subfig.pack_start(self.constrcanvas, True, True, 0)
        self.constrcanvas.show()

        self.window.show()

        self.mlist.addMagnet()

    def main(self):
        gtk.main()


if __name__=="__main__":
    hello = MagnetFun()
    hello.main()

